import * as postcss from 'postcss';
import selectorParser = require('postcss-selector-parser');
import { SelectorFactory, ParsedSelector, CompoundSelector } from "opticss";
import { Attribute, Attr } from "@opticss/element-analysis";
import { ObjectDictionary, MultiMap } from "@opticss/util";
import { NodeAndType } from "../BlockParser";
import { FileIdentifier } from "../importing";
import { OptionsReader } from "../OptionsReader";
import { LocalScopedContext, HasLocalScope, HasScopeLookup } from "../util/LocalScope";
export declare const OBJ_REF_SPLITTER: (s: string) => [string, string] | undefined;
/**
 * Abstract class that serves as the base for all BlockObjects. Contains basic
 * properties and abstract methods that extenders must implement.
 */
export declare abstract class BlockObject {
    readonly propertyConcerns: PropertyContainer;
    protected _name: string;
    protected _container: BlockParent;
    protected _compiledAttribute: Attribute;
    /** cache of resolveStyles() */
    private _resolvedStyles;
    /**
     * Save name, parent container, and create the PropertyContainer for this data object.
     */
    constructor(name: string, container?: BlockParent);
    /**
     * Readonly name of this object.
     */
    readonly name: string;
    /**
     * Block parent container.
     */
    readonly parent: BlockParent;
    /**
     * Get the inherited block object for this block object of the same name and type.
     * @returns The base inherited block object.
     */
    readonly abstract base: BlockObject | undefined;
    /**
     * Return the local identifier for this `BlockObject`.
     * @returns The local name.
     */
    abstract localName(): string;
    /**
     * Return an attribute for analysis using the authored source syntax.
     */
    abstract asSourceAttributes(): Attr[];
    /**
     * Return the source selector this `BlockObject` was read from.
     * @returns The source selector.
     */
    abstract asSource(): string;
    /**
     * Return the css selector for this `BlockObject`.
     * @param opts Option hash configuring output mode.
     * @returns The CSS class.
     */
    abstract cssClass(opts: OptionsReader): string;
    /**
     * Return true or false if the given `CompoundSelector`.
     * @param opts An options hash with an `outputMode` property of type `OutputMode` to switch output type.
     * @returns The CSS class.
     */
    abstract matches(compoundSel: CompoundSelector): boolean;
    /**
     * Returns the list of all `BlockObjects` that are ancestors of this block.
     */
    abstract all(): BlockObject[];
    /**
     * Crawl up the container tree and return the base block object.
     * @returns The base block in this container tree.
     */
    readonly block: Block;
    /**
     * Returns all the classes needed to represent this block object
     * including inherited classes.
     * @returns this object's css class and all inherited classes.
     */
    cssClasses(opts: OptionsReader): string[];
    /**
     * Return all Block Objects that are implied by this object.
     * This takes inheritance, state/class correlations, and any
     * other declared links between styles into account.
     *
     * This block object is included in the returned result so the
     * resolved value's size is always 1 or greater.
     */
    resolveStyles(): Set<BlockObject>;
    /**
     * Returns the styles that are directly implied by this style.
     * Does not include the styles that this style inherits implied.
     * Does not include the styles that this style implies inherits.
     *
     * returns undefined if no styles are implied.
     */
    impliedStyles(): Set<BlockObject> | undefined;
    /**
     * Compute all block objects that are implied by this block object through
     * inheritance. Does not include this object or the styles it implies through
     * other relationships to this object.
     *
     * If nothing is inherited, this returns an empty set.
     */
    resolveInheritance(): Set<BlockObject>;
    asCompiledAttributes(opts: OptionsReader): Attribute[];
    /**
     * Standard export method for a given block.
     * @param opts  Options for rendering cssClass.
     * @returns The Export object representing this BlockObject.
     */
    asExport(opts: OptionsReader): Export;
    /**
     * Debug utility to help log BlockObjects
     * @param opts  Options for rendering cssClass.
     * @returns A debug string.
     */
    asDebug(opts: OptionsReader): string;
}
export declare class Block extends BlockObject implements SelectorFactory, HasLocalScope<Block, BlockObject>, HasScopeLookup<BlockObject> {
    private _sourceAttribute;
    private _classes;
    private _blockReferences;
    private _identifier;
    private _base?;
    private _baseName?;
    private _implements;
    private _localScope;
    /**
     * array of paths that this block depends on and, if changed, would
     * invalidate the compiled css of this block. This is usually only useful in
     * preprocessed blocks.
     */
    private _dependencies;
    root?: postcss.Root;
    readonly states: StateContainer;
    readonly parsedRuleSelectors: WeakMap<postcss.Rule, ParsedSelector[]>;
    private hasHadNameReset;
    constructor(name: string, identifier: FileIdentifier);
    name: string;
    subScope(name: string): LocalScopedContext<Block, BlockObject> | undefined;
    lookupLocal(name: string): BlockObject | undefined;
    /**
     * Lookup a sub-block either locally, or on a referenced foreign block.
     * @param reference
     *   A reference to a block object adhering to the following grammar:
     *     reference -> <ident> '.' <sub-reference>  // reference through sub-block <ident>
     *                | <ident>                      // reference to sub-block <ident>
     *                | '.' <class-selector>         // reference to class in this block
     *                | <state-selector>             // reference to state in this block
     *                | '.'                          // reference to this block
     *     sub-reference -> <ident> '.' <sub-reference> // reference through sub-sub-block
     *                    | <object-selector>        // reference to object in sub-block
     *     object-selector -> <block-selector>       // reference to this sub-block
     *                      | <class-selector>       // reference to class in sub-block
     *                      | <state-selector>       // reference to state in this sub-block
     *     block-selector -> 'root'
     *     class-selector -> <ident>
     *     state-selector -> '[state|' <ident> ']'
     *     ident -> regex:[a-zA-Z_-][a-zA-Z0-9]*
     *   A single dot by itself returns the current block.
     * @returns The BlockObject referenced at the supplied path.
     */
    lookup(reference: string): BlockObject | undefined;
    asSourceAttributes(): Attribute[];
    /**
     * Add an absolute, normalized path as a compilation dependency. This is used
     * to invalidate caches and trigger watchers when those files change.
     *
     * It is not necessary or helpful to add css-block files.
     */
    addDependency(filepath: string): void;
    readonly dependencies: string[];
    readonly base: Block | undefined;
    readonly baseName: string | undefined;
    readonly identifier: FileIdentifier;
    setBase(baseName: string, base: Block): void;
    getClass(name: string): BlockClass | undefined;
    readonly implementsBlocks: Block[];
    addImplementation(b: Block): number;
    /**
     * Validate that this block implements all foreign selectors from blocks it impelemnts.
     * @param b The block to check implementation against.
     * @returns The BlockObjects from b that are missing in the block.
     */
    checkImplementation(b: Block): BlockObject[];
    /**
     * Validate that all foreign blocks this block implements are fully...implemented.
     */
    checkImplementations(): void;
    find(sourceName: string): BlockObject | undefined;
    eachBlockReference(callback: (name: string, block: Block) => any): void;
    readonly classes: BlockClass[];
    cssClass(opts: OptionsReader): string;
    localName(): string;
    addClass(blockClass: BlockClass): void;
    ensureClass(name: string): BlockClass;
    addBlockReference(localName: string, other: Block): void;
    getReferencedBlock(localName: string): Block | null;
    transitiveBlockDependencies(): Set<Block>;
    /**
     * Return array self and all children.
     * @param shallow Pass false to not include inherited objects.
     * @returns Array of BlockObjects.
     */
    all(shallow?: boolean): BlockObject[];
    merged(): MultiMap<string, BlockObject>;
    asSource(): string;
    /**
     * Fetch a the cached `BlockObject` from `Block` given `NodeAndType`.
     * @param obj The `NodeAndType` object to use for `BlockObject` lookup.
     */
    nodeAndTypeToBlockObject(obj: NodeAndType): BlockObject | undefined;
    nodeAsBlockObject(node: selectorParser.Node): [BlockObject, number] | null;
    rewriteSelectorNodes(nodes: selectorParser.Node[], opts: OptionsReader): selectorParser.Node[];
    rewriteSelectorToString(selector: ParsedSelector, opts: OptionsReader): string;
    rewriteSelector(selector: ParsedSelector, opts: OptionsReader): ParsedSelector;
    matches(compoundSel: CompoundSelector): boolean;
    debug(opts: OptionsReader): string[];
    /**
     * Test if the supplied block is the same block object.
     * @param other  The other Block to test against.
     * @return True or False if self and `other` are equal.
     */
    equal(other: Block | undefined | null): boolean | null | undefined;
    isAncestorOf(other: Block | undefined | null): boolean;
    /**
     * TODO: Move selector cache into `parseSelector` and have consumers of this
     *       method interface with the parseSelector utility directly.
     * Given a PostCSS Rule, ensure it is present in this Block's parsed rule
     * selectors cache, and return the ParsedSelector array.
     * @param rule  PostCSS Rule
     * @return ParsedSelector array
     */
    getParsedSelectors(rule: postcss.Rule): ParsedSelector[];
    /**
     * Objects that contain Blocks are often passed into assorted libraries' options
     * hashes. Some libraries like to `JSON.stringify()` their options to create
     * unique identifiers for re-run caching. (ex: Webpack, awesome-typescript-loader)
     * Blocks contain circular dependencies, so we need to override their `toJSON`
     * method so these libraries don't implode.
     * @return The name of the block.
     */
    toJSON(): string;
}
export declare function isBlock(o: object): o is Block;
/**
 * Cache and interface methods for block properties.
 */
export declare class PropertyContainer {
    private props;
    private pseudoProps;
    /**
     * Track a single property.
     * @param	property	The property we're tracking
     * @param	pseudo	The pseudo element this rule is styling, if applicable.
     */
    addProperty(property: string, pseudo?: string): void;
    /**
     * Track all properties from a ruleset in this block's PropertyContainer.
     * @param	rule	PostCSS ruleset
     * @param	block	External block
     */
    addProperties(rule: postcss.Rule, block: Block, filter?: (prop: string) => boolean): void;
    /**
     * Retreive properties from all rulesets in this block.
     * @param	pseudo	Optional pseudo element to get properties from
     * @returns A set of property names.
     */
    getProperties(pseudo?: string): Set<string>;
    /**
     * Retrieve the pseudo elements which were found to have properties.
     * @returns A set of property names.
     */
    getPseudos(): Set<string>;
}
/**
 * Holds state values to be passed to the StateContainer.
 */
export interface StateInfo {
    group?: string;
    name: string;
}
/**
 * Cache and interface methods for block states and state groups.
 */
export declare class StateContainer {
    private _states;
    private _groups;
    private _parent;
    /**
     * Save a reference to our parent element on instantiation. Only Blocks and
     * BlockClasses can contain States.
     */
    constructor(parent: Block | BlockClass);
    /**
     * Parent accessor.
     * @returns The parent object that contains these States
     */
    readonly parent: StateParent;
    /**
     * Insert a state into this container
     * @param state The State object to insert.
     * @param group Optional group name for this state object.
     * @returns The State that was just added
     */
    addState(state: State): State;
    /**
     * Group getter. Returns a list of State objects in the requested group.
     * @param group State group for lookup or a boolean state name if substate is not provided.
     * @param substate Optional substate to filter states by.
     * @returns An array of all States that were requested.
     */
    getGroup(groupName: string, substate?: string | undefined): State[];
    /**
     * like getGroup but includes the states from all super blocks for the group.
     * @param group State group for lookup
     * @param substate Optional substate to filter states by.
     * @returns A map of resolved state names to their states for all States that were requested.
     */
    resolveGroup(groupName: string, substate?: string | undefined): ObjectDictionary<State> | undefined;
    getGroups(): string[];
    getStates(group?: string): Array<State> | undefined;
    /**
     * State getter
     * @param name The State's name to lookup.
     * @param group  Optional state group for lookup
     * @returns The State that was requested, or undefined
     */
    getState(name: string, group?: string): State | undefined;
    /**
     * Legacy State getter
     * @param info The StateInfo type to lookup, contains `name` and `group`
     * @returns The State that was requested, or undefined
     */
    _getState(info: StateInfo): State | undefined;
    /**
     * Ensure that a `State` with the given `name` and `group` is registered with this Block.
     * @param name The State's name to ensure.
     * @param group  Optional state group for lookup/registration
     * @return The `State` object on this `Block`
     */
    ensureState(name: string, group?: string): State;
    /**
     * Legacy state ensurer. Ensure that a `State` with the given `StateInfo` is
     * registered with this Block.
     * @param info  `StateInfo` to verify exists on this `Block`
     * @return The `State` object on this `Block`
     */
    _ensureState(info: StateInfo): State;
    /**
     * Debug utility to help test StateContainer.
     * @param options  Options to pass to States' asDebug method.
     * @return Array of debug strings for these states
     */
    debug(opts: OptionsReader): string[];
    /**
     * Returns all states contained in this Container
     * @return Array of all State objects
     */
    all(): State[];
}
/**
 * Interface used when exporting a BlockObject using `asExport`.
 */
export interface Export {
    identifier: string;
    value: string;
}
/** Parent types for a state */
export declare type StateParent = Block | BlockClass;
/**
 * Valid parent types for a BlockObject
 */
export declare type BlockParent = StateParent | undefined;
/**
 * Represents a Class present in the Block.
 */
export declare class BlockClass extends BlockObject {
    private _sourceAttribute;
    readonly states: StateContainer;
    /**
     * BlockClass constructor
     * @param name Name for this BlockClass instance
     * @param parent The parent block of this class.
     */
    constructor(name: string, parent: Block);
    readonly base: BlockClass | undefined;
    localName(): string;
    /**
     * Export as original class name.
     * @returns String representing original class.
     */
    asSource(): string;
    asSourceAttributes(): Attribute[];
    /**
     * Export as new class name.
     * @param opts Option hash configuring output mode.
     * @returns String representing output class.
     */
    cssClass(opts: OptionsReader): string;
    /**
     * @returns Whether the given selector refers to this class
     */
    matches(compoundSel: CompoundSelector): boolean;
    /**
     * Return array self and all children.
     * @param shallow Pass false to not include children.
     * @returns Array of BlockObjects.
     */
    all(shallow?: boolean): BlockObject[];
}
export declare function isBlockClass(o: object): o is BlockClass;
/**
 * States represent a state attribute selector in a particular Block. States may
 * optionally be a member of a group of states, and or designated "global".
 */
export declare class State extends BlockObject {
    private _sourceAttributes;
    private _group;
    isGlobal: boolean;
    /**
     * State constructor. Provide a local name for this State, an optional group name,
     * and the parent container.
     * @param name The local name for this state.
     * @param group An optional parent group name.
     * @param container The parent container of this State.
     */
    constructor(name: string, group: string | null | undefined, container: Block | BlockClass);
    /**
     * Retrieve the BlockClass that this state belongs to, if applicable.
     * @returns The parent block class, or null.
     */
    readonly blockClass: BlockClass | null;
    /**
     * Retrieve this state's group name, if applicable.
     * @returns The parent group name, or null.
     */
    readonly group: string | null;
    unqualifiedSource(): string;
    /**
     * Retrieve this State's selector as it appears in the Block source code.
     * @returns The State's attribute selector
     */
    asSource(): string;
    asSourceAttributes(): Attr[];
    /**
     * Retrieve this State's local name, including the optional BlockClass and group designators.
     * @returns The State's local name.
     */
    localName(): string;
    /**
     * Export as new class name.
     * @param opts Option hash configuring output mode.
     * @returns String representing output class.
     */
    cssClass(opts: OptionsReader): string;
    /**
     * Given a StateInfo object, return whether this State object has the same group and name.
     * @param info StateInfo to compare against
     * @returns True or false.
     */
    private sameNameAndGroup(info);
    /**
     * @returns Whether the given selector refers to this state
     */
    matches(compoundSel: CompoundSelector): boolean;
    /**
     * Get the base inherited block object.
     * @returns The base inherited block object.
     */
    readonly base: State | undefined;
    /**
     * Return array self and all children.
     * @returns Array of BlockObjects.
     */
    all(): BlockObject[];
}
export declare function isState(o: object): o is State;
export declare function isSubState(o: object): o is State;
