import * as postcss from "postcss";
import { Block } from "../Block";
import { IBlockFactory } from "./IBlockFactory";
import BlockParser from "../BlockParser";
import { CssBlockOptionsReadonly } from "../options";
import { Importer, FileIdentifier, ImportedFile } from "../importing";
import { Preprocessors, Preprocessor } from "../preprocessing";
declare module "../options" {
    interface CssBlockOptions {
        /**
         * Depending on how css blocks are compiled, it may be necessary for a factory to be provided to the options
         * to ensure that instances are reused for compilation. Failing to do this can cause static analysis errors.
         */
        factory?: BlockFactory;
    }
}
/**
 * This factory ensures that instances of a block are re-used when blocks are
 * going to be compiled/optimized together. Multiple instances of the same
 * block will result in analysis and optimization bugs.
 *
 * This also ensures that importers and preprocessors are correctly used when loading a block file.
 */
export declare class BlockFactory implements IBlockFactory {
    postcssImpl: typeof postcss;
    importer: Importer;
    options: CssBlockOptionsReadonly;
    blockNames: {
        [name: string]: number;
    };
    parser: BlockParser;
    preprocessors: Preprocessors;
    private promises;
    private blocks;
    private paths;
    private preprocessQueue;
    constructor(options: CssBlockOptionsReadonly, postcssImpl?: typeof postcss);
    reset(): void;
    /**
     * In some cases (like when using preprocessors with native bindings), it may
     * be necessary to wait until the block factory has completed current
     * asynchronous work before exiting. Calling this method stops new pending
     * work from being performed and returns a promise that resolves when it is
     * safe to exit.
     */
    prepareForExit(): Promise<void>;
    getBlockFromPath(filePath: string): Promise<Block>;
    getBlock(identifier: FileIdentifier): Promise<Block>;
    _getBlockPromise(identifier: FileIdentifier): Promise<Block>;
    getBlockRelative(fromIdentifier: FileIdentifier, importPath: string): Promise<Block>;
    /**
     * Register a new block name with the BlockFactory. Return true true if successful, false if already exists.
     * @param name The new block name to register.
     * @return True or false depending on success status.
     */
    getUniqueBlockName(name: string): string;
    preprocessor(file: ImportedFile): Preprocessor;
}
