import * as postcss from 'postcss';
import selectorParser = require('postcss-selector-parser');
import { PluginOptions } from "./options";
import { Block, StateInfo } from "./Block";
import { IBlockFactory } from "./BlockFactory/IBlockFactory";
export { PluginOptions } from "./options";
import { SourceLocation } from "./SourceLocation";
import { CompoundSelector } from "opticss";
import { FileIdentifier } from "./importing";
import { Syntax } from "./preprocessing";
export declare const CLASS_NAME_IDENT: RegExp;
export declare enum BlockType {
    root = 1,
    state = 2,
    class = 3,
    classState = 4,
}
export interface NodeAndType {
    blockType: BlockType;
    node: selectorParser.Node;
}
export interface BlockNodeAndType extends NodeAndType {
    blockName?: string;
}
/**
 * Check if given selector node is targeting the root block node
 */
export declare function isRootNode(node: selectorParser.Node): boolean;
/**
 * Check if given selector node is a state selector
 * @param  node The selector to test.
 * @return True if state selector, false if not.
 */
export declare function isStateNode(node: selectorParser.Node): boolean;
/**
 * Check if given selector node is a class selector
 * @param  node The selector to test.
 * @return True if class selector, false if not.
 */
export declare function isClassNode(node: selectorParser.Node): boolean;
/**
 * CSS Blocks state parser.
 * @param  attr The css attribute selector that represents this state.
 * @return A `StateInfo` object that represents the state.
 */
export declare function stateParser(attr: selectorParser.Attribute): StateInfo;
export interface ParsedSource {
    identifier: FileIdentifier;
    defaultName: string;
    originalSource: string;
    originalSyntax: Syntax;
    parseResult: postcss.Result;
    dependencies: string[];
}
/**
 * Parser that, given a PostCSS AST will return a `Block` object. Main public
 * interface is `BlockParser.parse`.
 */
export default class BlockParser {
    private opts;
    private factory;
    constructor(opts: PluginOptions, factory: IBlockFactory);
    parseSource(source: ParsedSource): Promise<Block>;
    /**
     * Main public interface of `BlockParser`. Given a PostCSS AST, returns a promise
     * for the new `Block` object.
     * @param root  PostCSS AST
     * @param sourceFile  Source file name
     * @param defaultName Name of block
     */
    parse(root: postcss.Root, identifier: string, defaultName: string): Promise<Block>;
    private shouldBeParsedAsBlockSelector(rule);
    /**
     * Process all `@block-debug` statements, output debug statement to console or in comment as requested.
     * @param sourceFile File name of block in question.
     * @param root PostCSS Root for block.
     * @param block Block to resolve references for
     */
    private processDebugStatements(sourceFile, root, block);
    /**
     * For each `implements` property found in the passed ruleset, track the foreign
     * block. If block is not found, throw.
     * @param block  Block object being processed
     * @param sourceFile  Source file name, used for error output.
     * @param rule Ruleset to crawl
     */
    private implementsBlock(block, sourceFile, rule);
    /**
     * For each `extends` property found in the passed ruleset, set the block's base
     * to the foreign block. If block is not found, throw.
     * @param block  Block object being processed.
     * @param sourceFile  Source file name, used for error output.
     * @param rule Ruleset to crawl.
     */
    private extendBlock(block, sourceFile, rule);
    /**
     * Resolve all block references for a given block.
     * @param block Block to resolve references for
     * @return Promise that resolves when all references have been loaded.
     */
    private resolveReferences(block);
    /**
     * Assert that a provided state rule is valid.
     * @param sourceFile File name of block in question.
     * @param root The PostCSS Rule.
     * @param attr Attribute to verify.
     */
    private assertValidState(block, rule, attr);
    /**
     * Similar to assertBlockObject except it doesn't check for well-formedness
     * and doesn't ensure that you get a block object when not a legal selector.
     * @param sel The `CompoundSelector` to search.
     * @return Returns the block's name, type and node.
     */
    static getBlockNode(sel: CompoundSelector): BlockNodeAndType | null;
    /**
     * Parses a CompoundSelector and returns the discovered Block Object. Validates
     * well-formedness of the given selector in the process.
     * @param block The block that contains this selector we're validating.
     * @param sel The `CompoundSelector` in question.
     * @param rule The full `postcss.Rule` for nice error reporting.
     * @return Returns the block's name, type and node.
     */
    private assertBlockObject(block, sel, rule);
    /**
     * Internal method used to generate human readable error messages when parsing.
     * @param t The block type we're generating a human readable name for.
     * @param options Options for output, currently just to specify plurality.
     * @return A human readable descriptor for the given `BlockType`.
     */
    private objectTypeName(t, options?);
    /**
     * Test if the provided node representation is a root level object, aka: operating
     * on the root element.
     * @param object The CompoundSelector's descriptor object.
     */
    private isRootLevelObject(object);
    /**
     * Test if the provided node representation is a class level object, aka: operating
     * on an element contained by the root, not the root itself.
     * @param object The CompoundSelector's descriptor object.
     */
    private isClassLevelObject(object);
    /**
     * Verify that the external block referenced by `rule` selects on a state that
     * exists in the external block and is exposed as a global.
     * @param block The current block making the external reference.
     * @param rule The rule referencing the external block.
     * @param obj The parsed node making the external reference.
     */
    private assertForeignGlobalState(block, rule, obj);
    /**
     * Assert that a provided selector follows all the combinator rules required
     * of block declarations.
     * @param block The block in this selector belongs to.
     * @param rule The PostCSS Rule.
     * @param selector The `ParsedSelector` to verify.
     */
    private assertValidSelector(block, rule, selector);
    selectorSourceLocation(block: Block, rule: postcss.Rule, selector: selectorParser.Node): SourceLocation | undefined;
}
