import { OptionsReader as CssBlocksOptionsReader } from "../OptionsReader";
import { Block, BlockObject, State, StateParent, BlockClass } from "../Block";
import { ObjectDictionary } from "@opticss/util";
import { Element, SourceLocation } from "@opticss/element-analysis";
export interface HasState<Style extends State | number = State> {
    state: Style;
}
export declare function isBooleanState(o: object): o is HasState<State | number>;
export interface HasGroup<Style extends State | number = State> {
    group: ObjectDictionary<Style>;
}
export declare function isStateGroup(o: object): o is HasGroup<State | number>;
/**
 * A boolean condition.
 */
export interface Conditional<BooleanExpression> {
    condition: BooleanExpression;
}
export declare function isConditional(o: object): o is Conditional<any>;
/**
 * A string expression used to switch between states.
 * When falsy is allowed, it's possible to disable the states.
 */
export interface Switch<StringExpression> {
    stringExpression: StringExpression;
    /** whether the states can be disabled. */
    disallowFalsy?: boolean;
}
export declare function isSwitch(o: object): o is Switch<any>;
/**
 * When the style container is dynamic but the style itself is not.
 */
export interface Dependency<Container extends StateParent | number = StateParent> {
    container: Container;
}
export declare function hasDependency(o: object): o is Dependency<StateParent | number>;
/**
 * the main branch of the style-if helper and the else branch of the
 * style-unless helper
 */
export interface TrueCondition<Container extends StateParent | number = StateParent> {
    whenTrue: Array<Container>;
}
export declare function isTrueCondition(o: object): o is TrueCondition<StateParent | number>;
/**
 * the else branch of the style-if helper and the main branch of the
 * style-unless helper
 */
export interface FalseCondition<Container extends StateParent | number = StateParent> {
    whenFalse: Array<Container>;
}
export declare function isFalseCondition(o: object): o is FalseCondition<StateParent | number>;
/** A state that is conditionally set */
export declare type ConditionalState<BooleanExpression> = Conditional<BooleanExpression> & HasState;
/** A state that is only set when its dynamic class is set */
export declare type DependentState = Dependency & HasState;
/** A state that is only set when its condition is true and its dynamic class is set */
export declare type ConditionalDependentState<BooleanExpression> = Conditional<BooleanExpression> & Dependency & HasState;
/** A state that is dynamic for any reason */
export declare type DynamicState<BooleanExpression> = ConditionalState<BooleanExpression> | DependentState | ConditionalDependentState<BooleanExpression>;
/** A group of states where one is set conditionally */
export declare type ConditionalStateGroup<StringExpression> = Switch<StringExpression> & HasGroup;
/** A group of states that are only set when its dynamic class is set and where one (or none) is selected at runtime. */
export declare type ConditionalDependentStateGroup<StringExpression> = Switch<StringExpression> & Dependency & HasGroup;
/** A group of states that are dynamic for any reason */
export declare type DynamicStateGroup<StringExpression> = ConditionalStateGroup<StringExpression> | ConditionalDependentStateGroup<StringExpression>;
/** Any type of dynamic state or group of states. */
export declare type DynamicStates<BooleanExpression, StringExpression> = DynamicState<BooleanExpression> | DynamicStateGroup<StringExpression>;
/** a ternary expression where different classes can be set when true or false */
export declare type DynamicClasses<TernaryExpression> = (Conditional<TernaryExpression> & TrueCondition) | (Conditional<TernaryExpression> & FalseCondition) | (Conditional<TernaryExpression> & TrueCondition & FalseCondition);
export declare type SerializedConditionalState = Conditional<true> & HasState<number>;
export declare type SerializedDependentState = Dependency<number> & HasState<number>;
export declare type SerializedConditionalDependentState = Conditional<true> & Dependency<number> & HasState<number>;
export declare type SerializedDynamicState = SerializedConditionalState | SerializedDependentState | SerializedConditionalDependentState;
export declare type SerializedConditionalStateGroup = Switch<true> & HasGroup<number>;
export declare type SerializedDependentStateGroup = Dependency<number> & HasGroup<number>;
export declare type SerializedConditionalDependentStateGroup = Switch<true> & Dependency<number> & HasGroup<number>;
export declare type SerializedDynamicStateGroup = SerializedConditionalStateGroup | SerializedDependentStateGroup | SerializedConditionalDependentStateGroup;
export declare type SerializedDynamicContainer = Conditional<true> & (TrueCondition<number> | FalseCondition<number> | (TrueCondition<number> & FalseCondition<number>));
export declare type SerializedDynamicStates = SerializedDynamicState | SerializedDynamicStateGroup;
export interface SerializedElementAnalysis {
    id?: string | undefined;
    tagName?: string | undefined;
    sourceLocation?: SourceLocation;
    staticStyles: Array<number>;
    dynamicClasses: Array<SerializedDynamicContainer>;
    dynamicStates: Array<SerializedDynamicStates>;
}
/**
 * This class is used to track the styles and dynamic expressions on an element
 * and produce a runtime class expression in conjunction with a style mapping.
 *
 * This class tracks dependencies between states and classes and the runtime
 * expression causes states to be removed if the parent class is not present at
 * runtime.
 *
 * With some syntaxes, if an element has different classes on it at runtime
 * from the same block, the states in use become ambiguous. To manage this,
 * the caller must ensure that the state names used exist for all possible
 * classes and produce an error if not. Then add the states for each
 * possible class with multiple calls to add(Static|Dynamic)(State|Group).
 * If the AST can't handle multiple references pointing at the expression node,
 * the caller must clone it -- This won't result in multiple expression
 * invocations unless multiple classes from the same block are applied at the
 * same time -- which would be illegal.
 */
export declare class ElementAnalysis<BooleanExpression, StringExpression, TernaryExpression> {
    static autoGenerateId: symbol;
    /** an opaque id assigned from the analyzer for later retrieval */
    id: string | undefined;
    /** The name of the tag these styles are applied -- if known. */
    tagName: string | undefined;
    /** where the element starts and ends */
    sourceLocation: SourceLocation;
    /** static styles explicitly declared on this element */
    static: Set<BlockObject>;
    /** blocks/classes set conditionally */
    dynamicClasses: Array<DynamicClasses<TernaryExpression>>;
    /** states set dynamically or depending on a dynamic class */
    dynamicStates: Array<DynamicStates<BooleanExpression, StringExpression>>;
    private addedStyles;
    /** classes declared explicitly and found in at least one dynamic class expression. */
    private dynamicClassExpressions;
    /** All the classes on this element, by block. */
    private allClasses;
    /**
     * All the static styles including styles implied by the explicitly specified
     * styles.
     */
    private allStaticStyles;
    private _sealed;
    /** whether all styles have been added and the styles can be analyzed now. */
    readonly sealed: boolean;
    constructor(location: SourceLocation, tagName?: string, id?: string);
    hasStyles(): boolean;
    /**
     * Get a list of all possible block objects for the given block
     * on this element that can be used a parent for a state.
     *
     * This can be called before or after being sealed.
     */
    classesForBlock(block: Block): Array<StateParent>;
    /**
     * Checks if the given class or block is set on this element
     * of if it is implied by one of the other styles on this element.
     *
     * This can be called before or after being sealed.
     */
    hasClass(klass: StateParent): boolean;
    /**
     * iterate over all static and dynamic States explicitly set on this element
     *
     * the analysis must be sealed to call this method.
     * @param dynamic
     *   * undefined - return all states,
     *   * true - return only dynamic states
     *   * false - return only static states
     */
    statesFound(dynamic?: boolean): IterableIterator<State>;
    /**
     * iterate over all static and dynamic blocks and classes explicitly set on this element.
     *
     * the analysis must be sealed to call this method.
     * @param dynamic
     *   * undefined - return all classes,
     *   * true - return only dynamic classes
     *   * false - return only static classes
     */
    classesFound(dynamic?: boolean): IterableIterator<StateParent>;
    /**
     * This method indicates that all styles have been added
     * and can be analyzed and validated.
     */
    seal(): void;
    /**
     * This is used to add any static state even if it is part of a group.
     * The state is added as dynamic and conditional on its class if that
     * class is dynamic.
     */
    addStaticState(container: Block | BlockClass, state: State): void;
    private _addStaticState(style);
    private assertValidContainer(container, state);
    /**
     * Adds a state that is toggled on and off at runtime.
     *
     * @param state the state that is dynamic.
     * @param condition The AST node(s) representing this boolean expression.
     */
    addDynamicState(container: Block | BlockClass, state: State, condition: BooleanExpression): void;
    private _addDynamicState(style);
    /**
     * Adds a group of states that are toggled between at runtime.
     *
     * @param container The class or block root to which the states belong.
     * @param group The states that are to be chosen from. All must be sub states
     *   of the same group -- they can be from different blocks if they inherit.
     * @param condition The AST node(s) representing this group -- can be used
     *   by the rewriter. This is just an opaque value that is passed through.
     * @param disallowFalsy Whether a missing value is expected or should be
     *   a runtime error.
     */
    addDynamicGroup(container: StateParent, group: ObjectDictionary<State>, stringExpression: StringExpression, disallowFalsy?: boolean): void;
    private _addDynamicGroup(style);
    /**
     * Adds a state parent that is always set on this element.
     */
    addStaticClass(klass: StateParent): void;
    private _addStaticClass(style);
    /**
     * this maps the class and all the classes that the explicit class implies
     * to all the blocks those classes belong to via inheritance.
     *
     * These classes become valid containers for states even if they are not
     * explicitly set on the element.
     */
    private mapBlocksForClass(klass);
    /**
     * Adds state parents that are set based on a conditional.
     * This is modeled as a ternary (if/else) expression.
     *
     * Nested ternaries are not supported at this time.
     */
    addDynamicClasses(dynClasses: DynamicClasses<TernaryExpression>): void;
    private _addDynamicClasses(dynClasses);
    assertSealed(isSealed?: boolean): void;
    countAllStaticStyles(): number;
    getAllStaticStyles(): IterableIterator<BlockObject>;
    /**
     * Get a simple object with no circular references that is possible to
     * emit and restore as JSON.
     *
     * @param styleIndexes a map of block objects to a number that represents
     *   it in the template analysis serialization.
     */
    serialize(styleIndexes: Map<BlockObject, number>): SerializedElementAnalysis;
    /**
     * Compute an OptiCSS element description and a map of class names back to
     * the block objects for use in re-writing.
     *
     * This mapping of classname to block object is stable and the keys can be
     * assumed to be unique per BlockObject across all blocks -- so these
     * maps can be merged safely.
     */
    forOptimizer(options: CssBlocksOptionsReader): [Element, Map<string, BlockObject>];
    /**
     * Because of the possibility for classes to inherit and imply
     * other classes, we need to convert any dynamic class
     * that is static because of another static class to be static
     * This ensures that otherwise static states won't get
     * hoisted into dynamic expressions because of a class dependency.
     */
    private prepareForStates();
    private prepareCondition(classes, condition);
}
