import { OptionsReader } from '../OptionsReader';
import { BlockFactory } from "../BlockFactory";
import { StyleAnalysis } from "./StyleAnalysis";
import { BlockObject, Block } from "../Block";
import TemplateValidator, { TemplateValidatorOptions } from "./validations";
import { TemplateTypes, SerializedTemplateInfo, TemplateInfo, TemplateAnalysis as OptimizationTemplateAnalysis, TemplateIntegrationOptions } from "@opticss/template-api";
import { SourceLocation, SourcePosition } from "@opticss/element-analysis";
import { ObjectDictionary } from "@opticss/util";
import { ElementAnalysis, SerializedElementAnalysis } from "./ElementAnalysis";
import { IdentGenerator } from "opticss";
/**
 * This interface defines a JSON friendly serialization
 * of a {TemplateAnalysis}.
 */
export interface SerializedTemplateAnalysis<K extends keyof TemplateTypes> {
    template: SerializedTemplateInfo<K>;
    blocks: ObjectDictionary<string>;
    stylesFound: string[];
    elements: ObjectDictionary<SerializedElementAnalysis>;
}
/**
 * A TemplateAnalysis performs book keeping and ensures internal consistency of the block objects referenced
 * within a template. It is designed to be used as part of an AST walk over a template.
 *
 * 1. Call [[startElement startElement()]] at the beginning of an new html element.
 * 2. Call [[addStyle addStyle(blockObject, isDynamic)]] for all the styles used on the current html element.
 * 2. Call [[addExclusiveStyle addExclusiveStyle(alwaysPresent, ...blockObject)]] for all the styles used that are mutually exclusive on the current html element.
 * 3. Call [[endElement endElement()]] when done adding styles for the current element.
 */
export declare class TemplateAnalysis<K extends keyof TemplateTypes> implements StyleAnalysis {
    template: TemplateInfo<K>;
    idGenerator: IdentGenerator;
    /**
     * A map from a local name for the block to the [[Block]].
     * The local name must be a legal CSS ident/class name but this is not validated here.
     * See [[CLASS_NAME_IDENT]] for help validating a legal class name.
     */
    blocks: ObjectDictionary<Block>;
    /**
     * Return the number of blocks discovered in this Template.
     */
    blockCount(): number;
    /**
     * Return the number of styles discovered in this Analysis' Template.
     * This is slow.
     */
    styleCount(): number;
    /**
     * A per-element correlation of styles used. The current correlation is added
     * to this list when [[endElement]] is called.
     */
    elements: Map<string, ElementAnalysis<any, any, any>>;
    /**
     * The current element, created when calling [[startElement]].
     * The current element is unset after calling [[endElement]].
     */
    currentElement: ElementAnalysis<any, any, any> | undefined;
    /**
     * Return the number of elements discovered in this Analysis.
     */
    elementCount(): number;
    /**
     * Get the nth element discovered in this Analysis.
     */
    getElement<BooleanExpression, StringExpression, TernaryExpression>(idx: number): ElementAnalysis<BooleanExpression, StringExpression, TernaryExpression>;
    optimizationOptions(): TemplateIntegrationOptions;
    /**
     * Get an Element by ID.
     */
    getElementById<BooleanExpression, StringExpression, TernaryExpression>(id: string): ElementAnalysis<BooleanExpression, StringExpression, TernaryExpression> | undefined;
    /**
     * Template validator instance to verify blocks applied to an element.
     */
    validator: TemplateValidator;
    /**
     * @param template The template being analyzed.
     */
    constructor(template: TemplateInfo<K>, options?: TemplateValidatorOptions);
    /**
     * @param block The block for which the local name should be returned.
     * @return The local name of the given block.
     */
    getBlockName(block: Block): string | null;
    startElement<BooleanExpression, StringExpression, TernaryExpression>(location: SourceLocation | SourcePosition, tagName?: string): ElementAnalysis<BooleanExpression, StringExpression, TernaryExpression>;
    endElement<BooleanExpression, StringExpression, TernaryExpression>(element: ElementAnalysis<BooleanExpression, StringExpression, TernaryExpression>, endPosition?: SourcePosition): void;
    private addFilename(pos);
    addElement<BooleanExpression, StringExpression, TernaryExpression>(element: ElementAnalysis<BooleanExpression, StringExpression, TernaryExpression>): void;
    /**
     * @return The local name for the block object using the local prefix for the block.
     */
    serializedName(o: BlockObject): string;
    /**
     * All the blocks referenced by this analysis.
     */
    referencedBlocks(): Block[];
    /**
     * All the blocks referenced by this block and all the blocks they reference recursively.
     */
    transitiveBlockDependencies(): Set<Block>;
    /**
     * All bhe blocks this block depends on. Same as referenced blocks except for the return type.
     */
    blockDependencies(): Set<Block>;
    stylesFound(dynamic?: boolean): IterableIterator<BlockObject>;
    /**
     * Generates a [[SerializedTemplateAnalysis]] for this analysis.
     */
    serialize(): SerializedTemplateAnalysis<K>;
    /**
     * Creates a TemplateAnalysis from its serialized form.
     * @param serializedAnalysis The analysis to be recreated.
     * @param options The plugin options that are used to parse the blocks.
     * @param postcssImpl The instance of postcss that should be used to parse the block's css.
     */
    static deserialize<K extends keyof TemplateTypes>(serializedAnalysis: SerializedTemplateAnalysis<K>, blockFactory: BlockFactory): Promise<TemplateAnalysis<K>>;
    forOptimizer(opts: OptionsReader): OptimizationTemplateAnalysis<K>;
}
