import { CssBlockOptionsReadonly } from "./options";
import { Syntax } from "./preprocessing";
declare module "./options" {
    interface CssBlockOptions {
        importer: Importer;
        data: ImporterData;
    }
}
export interface ImporterData {
    [key: string]: any;
}
/**
 * A FileIdentifier is a string with a whatever internal encoding is needed to uniquely resolve
 * a file or relative importPath against the identifier by an importer. FileIdentifiers may be
 * serialized across processes and should not encode any transient state. If an importer
 * wraps another importer, it is responsible for mangling and demangling the import identifier to
 * ensure that the namespaces of the importers do not collide.
 *
 * Care should be taken to ensure that the same block file is never returned
 * with different identifiers. The identifier a returned on an ImportedFile
 * should be different from the identifier that was requested if the requested
 * identifier was not canonical. The block factory will ensure that all blocks
 * returned to the consumer are unique to the canonical identifier.
 */
export declare type FileIdentifier = string;
/**
 * Structure that CSS Blocks uses to represent a single file.
 */
export interface ImportedFile {
    /**
     * A unique identifier (probably an absolute filesystem path) that describes
     * the block and can be used for caching.
     */
    identifier: FileIdentifier;
    /**
     * The default name for the block based on its identifier. This is used when a block doesn't specify a name for itself.
     */
    defaultName: string;
    /**
     * The syntax of the source contents. This could be determined by filename extension or some other metadata.
     */
    syntax: Syntax;
    /**
     * The contents of the imported file.
     */
    contents: string;
}
/**
 * Interface for supported CSS Blocks file importers. All custom importers must
 * implement this shape.
 */
export interface Importer {
    /**
     * compute a unique identifier for a given import path. If `fromIdentifier` is provided,
     * the importPath can be relative to the file that is identified by it.
     */
    identifier(fromIdentifier: FileIdentifier | null, importPath: string, options: CssBlockOptionsReadonly): FileIdentifier;
    /**
     * import the file with the given metadata and return a string and meta data for it.
     */
    import(identifier: FileIdentifier, options: CssBlockOptionsReadonly): Promise<ImportedFile>;
    /**
     * the default name of the block used unless the block specifies one itself.
     */
    defaultName(identifier: FileIdentifier, options: CssBlockOptionsReadonly): string;
    /**
     * If a file identifier has an on-disk representation, return an absolute path to it.
     */
    filesystemPath(identifier: FileIdentifier, options: CssBlockOptionsReadonly): string | null;
    /**
     * Returns a string meant for human consumption that identifies the file.
     * As is used in debug statements and error reporting. Unlike filesystemPath,
     * this needn't resolve to an actual file or be an absolute path.
     */
    debugIdentifier(identifier: FileIdentifier, options: CssBlockOptionsReadonly): string;
    /**
     * returns the syntax the contents are written in.
     */
    syntax(identifier: FileIdentifier, options: CssBlockOptionsReadonly): Syntax;
}
export declare abstract class PathBasedImporter implements Importer {
    identifier(fromFile: string | null, importPath: string, options: CssBlockOptionsReadonly): string;
    defaultName(identifier: string, _options: CssBlockOptionsReadonly): string;
    filesystemPath(identifier: FileIdentifier, _options: CssBlockOptionsReadonly): string | null;
    syntax(identifier: FileIdentifier, options: CssBlockOptionsReadonly): Syntax;
    debugIdentifier(identifier: FileIdentifier, options: CssBlockOptionsReadonly): string;
    abstract import(identifier: FileIdentifier, options: CssBlockOptionsReadonly): Promise<ImportedFile>;
}
export declare class FilesystemImporter extends PathBasedImporter {
    filesystemPath(identifier: FileIdentifier, _options: CssBlockOptionsReadonly): string | null;
    import(identifier: FileIdentifier, options: CssBlockOptionsReadonly): Promise<ImportedFile>;
}
export interface Alias {
    /**
     * A path segment that identifies this path location. Relative import paths can start with this
     * path segment and will be resolved to the corresponding absolute path.
     */
    alias: string;
    /**
     * An absolute path that the alias refers to.
     */
    path: string;
}
export declare type PathAliases = Alias[] | {
    /**
     * map of alias prefixes to absolute paths.
     */
    [alias: string]: string;
};
/**
 * The PathAliasImporter is a replacement for the fileystem importer. Relative import paths
 * are first checked to see if they match an existing file relative to the from identifier (when provided).
 * Then if the relative import path has a first segment that is any of the aliases provided the path
 * will be made absolute using that alias's path location. Finally any relative path is resolved against
 * the rootDir specified from {CssBlockOptionsReadonly}.
 *
 * When debugging an identifier it is made relative to an alias, if one exists, where the identifier is
 * within an aliased directory. If several such aliased paths exist, the most specific alias will be used.
 */
export declare class PathAliasImporter extends FilesystemImporter {
    aliases: Alias[];
    constructor(aliases: PathAliases);
    identifier(from: FileIdentifier | null, importPath: string, options: CssBlockOptionsReadonly): string;
    debugIdentifier(identifier: FileIdentifier, options: CssBlockOptionsReadonly): string;
}
/**
 * Default importer. Returns `ImportedFile` from disk
 * @param fromFile
 * @param importPath
 */
export declare let filesystemImporter: FilesystemImporter;
